// ----------------------------------------------------------------------------
// CounterExample.java
// The "counterexample" command.
// $Id: CounterExample.java,v 1.2 2006/03/29 15:49:49 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import java.io.*;
import org.w3c.dom.*;
import fmrisc.Proving.*;
import fmrisc.ProofNavigator.*;

/**
 * 
 * Command "counterexample": show potential counterexample for current
 * state (may be time-consuming)
 *
 */
public final class CounterExample extends ProofCommandBase
{
  public final static String Name = "counterexample";
  
  /***************************************************************************
   * Create a "counterexample" command.
   **************************************************************************/
  public CounterExample()
  {
    super(Name);
  }
  
  /****************************************************************************
   * Process command "counterexample": 
   * @param state the proof state to which the command is applied
   * @return null or array of single new proof state.
   ***************************************************************************/
  protected ProofState[] processCore(ProofState state)
  {
    // auxiliary state as duplicate of current state
    ProofState state0 = new ProofState(state, state.getEnvironment());
    state0.setAssumptions(state0.copy(state.getAssumptions()));
    state0.setGoals(state0.copy(state.getGoals()));
    
    // no hard simplification but generate counterexample for auxiliary state
    boolean okay = ProverReasoning.simplify(state0, false, true);
    if (!okay) return null;
    
    // set counterexample in old state and update presentation
    state.setCounterExample(state0.getCounterExample());
    Main.setProofState(state);
    state.print(State.getSystemOut());
    
    // do not generate new state
    return null;
  }
  
  /***************************************************************************
   * Convert DOM node to command.
   * @param node the DOM representation of the command.
   * @return the command (null, if an error occured).
   **************************************************************************/
  public static Command newCommand(Node node)
  {
    return new CounterExample();
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
  }
}
