// ----------------------------------------------------------------------------
// TypeC.java
// The "type" command.
// $Id: TypeC.java,v 1.7 2006/02/04 20:20:58 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import java.io.PrintWriter;

import fmrisc.Communication.PrettyPrinter;
import fmrisc.ProofNavigator.State;
import fmrisc.Semantics.*;
import fmrisc.Syntax.*;

/**
 * 
 * Command "type": print value of type identifier.
 *
 */
public final class TypeC extends CommandBase
{
  public final static String Name = "type";
  private Identifier ident;
  
  /***************************************************************************
   * Create a "type" command.
   * @param ident identifier of denoted type
   **************************************************************************/
  public TypeC(Identifier ident)
  {
    super(Name);
    this.ident = ident;
  }
  
  /****************************************************************************
   * Process command "type": print value of type identifier.
   ***************************************************************************/
  public void process()
  { 
    Environment env = getEnvironment();
    TypeSymbol symbol = env.getTypeSymbol(ident);
    if (symbol == null)
    {
      error("there is no type named " + ident.toString() + 
              " visible in current environment");
      return;
    }
    PrintWriter out = State.getSystemOut();
    PrettyPrinter printer = new PrettyPrinter(out);
    String name = ident.toString();
    printer.print(name);
    printer.print(": TYPE");
    Type type = symbol.getType();
    if (type instanceof AtomicType)
    {
      AtomicType type0 = (AtomicType)type;
      if (name.equals(type0.getName()))
      {
        out.println(".");
        return;
      }
    }
    printer.print(" = ");
    printer.changeLevel(true);
    printer.newLine();
    printer.print(type);
    out.println(".");
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
    out.print(" ");
    ident.print(out);
  }
}