// ----------------------------------------------------------------------------
// ExpressionBase.java
// Base class of all expressions (terms and formulas)
// $Id: ExpressionBase.java,v 1.4 2005/11/02 14:23:18 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.Syntax;

import java.io.*;

/**
 * 
 * Base class of all expressions (terms and formulas)
 *
 */
public abstract class ExpressionBase extends ASTBase implements Expression
{  
  /***************************************************************************
   * get binding level for printing (lower numbers mean greater binding power)
   * @return the priority level
   **************************************************************************/
  public abstract int getPriority();
  
  /****************************************************************************
   * print expression, use parentheses if priority is greater than treshold
   * @param treshold priority level of enclosing expression
   * @param out writer to print expression on
   ***************************************************************************/
  public void printPriority(int treshold, PrintWriter out)
  {
    int p = getPriority();
    if (p > treshold) out.print("(");
    print(out);
    if (p > treshold) out.print(")");
  }
}
