// ----------------------------------------------------------------------------
// RecordType.java
// Handling of record types.
// $Id: RecordType.java,v 1.10 2009/12/03 15:43:09 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.Syntax;

import java.io.*;

/**
 * Handling of record types.
 */
public final class RecordType extends TypeBase
{
  // the record type components
  private TypedIdentifier[] base;
 
  /****************************************************************************
   * construct record type
   * @param base sequence of <identifier:type> pairs
   ***************************************************************************/
  public RecordType(TypedIdentifier[] base)
  {
    this.base = base;
  }
  
  /****************************************************************************
   * returns base types
   * @return the base types
   ***************************************************************************/
  public TypedIdentifier[] getBase()
  {
    return base;
  }
  
  /***************************************************************************
   * return copy of AST with value references instantiated by the substitutions
   * set in the corresponding declaration identifiers
   * @return the instantiated copy
   **************************************************************************/
  public Type instantiateType()
  {
    TypedIdentifier[] base = getBase();
    int n = base.length;
    TypedIdentifier[] base0 = new TypedIdentifier[n];
    for (int i=0; i<n; i++)
      base0[i] = base[i].instantiate();
    return Construct.recordType(base0);
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    int n = base.length;
    out.print("[#");
    for (int i=0; i<n-1; i++)
    {
      base[i].print(out);
      out.print(", ");
    }
    if (n>0) base[n-1].print(out);
    out.print("#]");
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be null)
   **************************************************************************/
  public AST accept(ASTVisitor visitor) 
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    int n = base.length;
    TypedIdentifier[] base0 = new TypedIdentifier[n];
    for (int i=0; i<n; i++)
      base0[i] = (TypedIdentifier)base[i].accept(visitor);
    if (!visitor.isCloned()) return null;
    return Construct.recordType(base0);
  }
}