// ----------------------------------------------------------------------------
// SubrangeType.java
// Handling of subtypes.
// $Id: SubrangeType.java,v 1.9 2006/02/23 19:51:05 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.Syntax;

import java.io.*;

/**
 * 
 * Subranges of integers.
 *
 */
public final class SubrangeType extends TypeBase
{
  private Expression from;  // the lower bound
  private Expression to;    // the upper bound

  /****************************************************************************
   * construct subrangetype with denoted bounds
   * @param from the lower bound
   * @param to the upper bound
   ***************************************************************************/
  public SubrangeType(Expression from, Expression to)
  {
    this.from = from;
    this.to = to;
  }
  
  /****************************************************************************
   * returns lower bound
   * @return the lower bound of the subrange
   ***************************************************************************/
  public Expression getFrom()
  {
    return from;
  }
  
  /****************************************************************************
   * returns to bound
   * @return the upper bound of the subrange
   ***************************************************************************/
  public Expression getTo()
  {
    return to;
  }
  
  /***************************************************************************
   * return copy of AST with value references instantiated by the substitutions
   * set in the corresponding declaration identifiers
   * @return the instantiated copy
   **************************************************************************/
  public Type instantiateType()
  {
    return Construct.subrangeType(getFrom().instantiate(),
      getTo().instantiate());
  }
  
  /***************************************************************************
   * Prints text representation of type on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print("[");
    from.printCore(out);
    out.print("..");
    to.printCore(out);
    out.print("]");
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be null)
   **************************************************************************/
  public AST accept(ASTVisitor visitor) 
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    AST from0 = from.accept(visitor);
    AST to0 = to.accept(visitor);
    if (!visitor.isCloned()) return null;
    return Construct.subrangeType((Expression)from0, (Expression)to0);
  }
}
